package com.caliverse.admin.domain.dao.admin;

import com.caliverse.admin.domain.entity.Item;
import com.caliverse.admin.domain.entity.Mail;
import com.caliverse.admin.domain.entity.Message;
import com.caliverse.admin.domain.request.MailRequest;

import java.util.List;
import java.util.Map;


public interface MailMapper {

    List<Mail> getMailList(Map map);
    int getAllCnt(Map map);
    int getTotal();
    Mail getMailDetail(Long id);

    List<Message> getMessage(Long id);

    List<Item> getItem(Long id);
    void postMail(MailRequest mailRequest);
    void insertGuid(Map map);

    void insertMessage(Map map);
    void insertItem(Map map);
    int updateMail(MailRequest mailRequest);

    int deleteMessage(Map map);

    int deleteItem(Map map);

    void deleteMail(Map map);

    List<Mail> getSchesuleMailList();

    int updateCompleteMail(Map map);

}
